@@ -258,9 +258,9 @@ def remove_lvm_debug():
 	# If we are running the lvmdbusd daemon and collecting lvm debug data, check and
 	# clean-up after the tests.
 	tmpdir = tempfile.gettempdir()
-	for f in glob("lvmdbusd.lvm.debug.*.log", root_dir=tmpdir):
-		fn = os.path.join(tmpdir, f)
-		os.unlink(fn)
+	fp = os.path.join(tmpdir, "lvmdbusd.lvm.debug.*.log")
+	for f in glob(fp):
+		os.unlink(f)
 
 
 class DaemonInfo(object):
@@ -446,7 +446,7 @@ class TestDbusService(unittest.TestCase):
 		self.handle_return(vg_proxy.Vg.Remove(dbus.Int32(g_tmo), EOD))
 		if is_nested_pv(pv_proxy.Pv.Name):
 			rc = self._pv_remove(pv_proxy)
-			self.assertTrue(rc == '/')
+			self.assertTrue(rc == '/', "We expected a '/', but got %s when removing a PV" % str(rc))
 
 	def clean_up(self):
 		self.objs, self.bus = get_objects()
@@ -522,7 +522,8 @@ class TestDbusService(unittest.TestCase):
 
 		self._validate_lookup(device, pv_path)
 
-		self.assertTrue(pv_path is not None and len(pv_path) > 0)
+		self.assertTrue(pv_path is not None and len(pv_path) > 0,
+						"When creating a PV we expected the returned path to be valid")
 		return pv_path
 
 	def _manager(self):
@@ -536,7 +537,7 @@ class TestDbusService(unittest.TestCase):
 
 	def test_version(self):
 		rc = self.objs[MANAGER_INT][0].Manager.Version
-		self.assertTrue(rc is not None and len(rc) > 0)
+		self.assertTrue(rc is not None and len(rc) > 0, "Manager.Version is invalid")
 		self._check_consistency()
 
 	def _vg_create(self, pv_paths=None, vg_prefix=None, options=None):
@@ -557,7 +558,7 @@ class TestDbusService(unittest.TestCase):
 				options))
 
 		self._validate_lookup(vg_name, vg_path)
-		self.assertTrue(vg_path is not None and len(vg_path) > 0)
+		self.assertTrue(vg_path is not None and len(vg_path) > 0, "During VG creation, returned path is empty")
 
 		intf = [VG_INT, ]
 		if self.vdo:
@@ -2301,13 +2302,15 @@ class TestDbusService(unittest.TestCase):
 			self._vdo_pool_lv(), test_ss=False)
 
 	def _log_file_option(self):
-		fn = "/tmp/%s" % rs(8, "_lvm.log")
+		fn = os.path.join(tempfile.gettempdir(), rs(8, "_lvm.log"))
 		try:
 			options = dbus.Dictionary({}, signature=dbus.Signature('sv'))
 			option_str = "log { level=7 file=%s syslog=0 }" % fn
 			options["config"] = dbus.String(option_str)
 			self._vg_create(None, None, options)
-			self.assertTrue(os.path.exists(fn))
+			self.assertTrue(os.path.exists(fn),
+							"We passed the following options %s to lvm while creating a VG and the "
+							"log file we expected to exist (%s) was not found" % (option_str, fn))
 		finally:
 			if os.path.exists(fn):
 				os.unlink(fn)
@@ -2344,6 +2347,9 @@ class TestDbusService(unittest.TestCase):
 
 	def test_z_sigint(self):
 		# Issue SIGINT while daemon is processing work to ensure we shut down.
+		if bool(int(os.getenv("LVM_DBUSD_TEST_SKIP_SIGNAL", "0"))):
+			raise unittest.SkipTest("Skipping as env. LVM_DBUSD_TEST_SKIP_SIGNAL is '1'")
+
 		di = DaemonInfo.get()
 		self.assertTrue(di is not None)
 		if di:
@@ -2410,7 +2416,8 @@ class TestDbusService(unittest.TestCase):
 	def _block_present_absent(self, block_device, present=False):
 		start = time.time()
 		keep_looping = True
-		while keep_looping and time.time() < start + 3:
+		max_wait = 10
+		while keep_looping and time.time() < start + max_wait:
 			if present:
 				if (self._lookup(block_device) != "/"):
 					keep_looping = False
@@ -2419,16 +2426,16 @@ class TestDbusService(unittest.TestCase):
 					keep_looping = False
 
 		if keep_looping:
-			print("Daemon failed to update")
+			print("Daemon failed to update within %d seconds!" % max_wait)
 		else:
 			print("Note: Time for udev update = %f" % (time.time() - start))
 		if present:
 			rc = self._lookup(block_device)
-			self.assertNotEqual(rc, '/')
+			self.assertNotEqual(rc, '/', "Daemon failed to update, missing udev change event?")
 			return True
 		else:
 			rc = self._lookup(block_device)
-			self.assertEqual(rc, '/')
+			self.assertEqual(rc, '/', "Daemon failed to update, missing udev change event?")
 			return True
 
 	def test_wipefs(self):
@@ -2497,11 +2504,55 @@ class TestDbusService(unittest.TestCase):
 			j.update()
 			if j.Complete:
 				(ec, error_msg) = j.GetError
-				self.assertTrue("insufficient free space" in error_msg, error_msg)
+				self.assertTrue("insufficient free space" in error_msg,
+								"We're expecting 'insufficient free space' in \n\"%s\"\n, stderr missing?" % error_msg)
 				break
 			else:
 				time.sleep(0.1)
 
+	@staticmethod
+	def _is_vg_devices_supported():
+		rc, stdout_txt, stderr_txt = call_lvm(["vgcreate", "--help"])
+		if rc == 0:
+			for line in stdout_txt.split("\n"):
+				if "--devices " in line:
+					return True
+		return False
+
+	@staticmethod
+	def _vg_create_specify_devices(name, device):
+		cmd = [LVM_EXECUTABLE, "vgcreate", "--devices", device, name, device]
+		outcome = Popen(cmd, stdout=PIPE, stderr=PIPE, close_fds=True, env=os.environ)
+		outcome.communicate()
+		if outcome.returncode == 0:
+			return True
+		else:
+			print("Failed to create vg %s, stdout= %s, stderr= %s" % (name, outcome.stdout, outcome.stderr))
+			return False
+
+	def test_duplicate_vg_name(self):
+		# LVM allows duplicate VG names, test handling renames for now
+		if not TestDbusService._is_vg_devices_supported():
+			raise unittest.SkipTest("lvm does not support vgcreate with --device syntax")
+
+		if len(self.objs[PV_INT]) < 2:
+			raise unittest.SkipTest("we need at least 2 PVs to run test")
+
+		vg_name = vg_n()
+		if TestDbusService._vg_create_specify_devices(vg_name, self.objs[PV_INT][0].Pv.Name) and \
+				TestDbusService._vg_create_specify_devices(vg_name, self.objs[PV_INT][1].Pv.Name):
+			objects, _ = get_objects()
+			self.assertEqual(len(objects[VG_INT]), 2)
+
+			if len(objects[VG_INT]) == 2:
+				for vg in objects[VG_INT]:
+					new_name = vg_n()
+					vg.Vg.Rename(dbus.String(new_name), dbus.Int32(g_tmo), EOD)
+					# Ensure we find the renamed VG
+					self.assertNotEqual("/", self._lookup(new_name), "Expecting to find VG='%s'" % new_name)
+		else:
+			self.assertFalse(True, "We failed to create 2 VGs with same name!")
+
 
 class AggregateResults(object):
 
@@ -2526,6 +2577,12 @@ if __name__ == '__main__':
 	r = AggregateResults()
 	mode = int(test_shell)
 
+	# To test with error injection, simply set the env. variable LVM_BINARY to the error inject script
+	# and the LVM_MAN_IN_MIDDLE variable to the lvm binary to test which defaults to "/usr/sbin/lvm"
+	# An example
+	# export LVM_BINARY=/home/tasleson/projects/lvm2/test/dbus/lvm_error_inject.py
+	# export LVM_MAN_IN_MIDDLE=/home/tasleson/projects/lvm2/tools/lvm
+
 	if mode == 0:
 		std_err_print('\n*** Testing only lvm fork & exec test mode ***\n')
 	elif mode == 1:
